/* 
-*- coding: latin-1 -*-

This file is part of RefactorErl.

RefactorErl is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

RefactorErl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with RefactorErl.  If not, see <http://plc.inf.elte.hu/erlang/>.

The Original Code is RefactorErl.

The Initial Developer of the Original Code is Eötvös Loránd University.
Portions created  by Eötvös Loránd University and ELTE-Soft Ltd.
are Copyright 2007-2013 Eötvös Loránd University, ELTE-Soft Ltd.
and Ericsson Hungary. All Rights Reserved.
*/

package com.refactorerl.ui.presentation.filelist;

import java.util.Collection;

import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.core.resources.IFile;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import com.refactorerl.ui.logic.IsFilePresentRequest;
import com.refactorerl.ui.presentation.Activator;

/**
 * Tests whether a selected Workspace resource is in the RefactorErl database. 
 * This can be used to decide if RefactorErl plugin functionality should shown in
 * perspectives other than the RefactorErl perspective (eg. Erlide perspective).  
 * @author Daniel Lukacs, 2014 ELTE IK
 *
 */
public class ResourceInDatabasePropertyTester extends PropertyTester {

	@Override
	public boolean test(Object receiver, String property, Object[] args, Object expectedValue) {

		if (receiver instanceof Collection) {
			Collection<?> cReceiver = (Collection<?>) receiver;
			if (!cReceiver.isEmpty() && cReceiver.iterator().next() instanceof IFile) {
				IFile ifile = ((IFile) cReceiver.iterator().next());

				Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

				return Activator.executeRequest(new IsFilePresentRequest(ifile.getLocation().toPortableString(),
						Activator.getProxy(), null), shell, true);

			}
		}

		return false;
	}

}
